#include <stdexcept>
#include <windows.h>
#include <winioctl.h>


#define SPBTESTTOOL_NAME L"i2c_bridge"
#define SPBTESTTOOL_SYMBOLIC_NAME L"\\DosDevices\\" SPBTESTTOOL_NAME
#define SPBTESTTOOL_USERMODE_PATH L"\\\\.\\" SPBTESTTOOL_NAME
#define SPBTESTTOOL_USERMODE_PATH_SIZE sizeof(SPBTESTTOOL_USERMODE_PATH)

#define FILE_DEVICE_SPB_PERIPHERAL 0x400

#define IOCTL_SPBTESTTOOL_OPEN              CTL_CODE(FILE_DEVICE_SPB_PERIPHERAL, 0x700, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_SPBTESTTOOL_CLOSE             CTL_CODE(FILE_DEVICE_SPB_PERIPHERAL, 0x701, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_SPBTESTTOOL_LOCK              CTL_CODE(FILE_DEVICE_SPB_PERIPHERAL, 0x702, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_SPBTESTTOOL_UNLOCK            CTL_CODE(FILE_DEVICE_SPB_PERIPHERAL, 0x703, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_SPBTESTTOOL_WRITEREAD         CTL_CODE(FILE_DEVICE_SPB_PERIPHERAL, 0x704, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_SPBTESTTOOL_LOCK_CONNECTION   CTL_CODE(FILE_DEVICE_SPB_PERIPHERAL, 0x705, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_SPBTESTTOOL_UNLOCK_CONNECTION CTL_CODE(FILE_DEVICE_SPB_PERIPHERAL, 0x706, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_SPBTESTTOOL_SIGNAL_INTERRUPT  CTL_CODE(FILE_DEVICE_SPB_PERIPHERAL, 0x707, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_SPBTESTTOOL_WAIT_ON_INTERRUPT CTL_CODE(FILE_DEVICE_SPB_PERIPHERAL, 0x708, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_SPBTESTTOOL_FULL_DUPLEX       CTL_CODE(FILE_DEVICE_SPB_PERIPHERAL, 0x709, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_SPBTESTTOOL_WRITE				CTL_CODE(FILE_DEVICE_SPB_PERIPHERAL, 0x70a, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define IOCTL_SPBTESTTOOL_READ				CTL_CODE(FILE_DEVICE_SPB_PERIPHERAL, 0x70b, METHOD_BUFFERED, FILE_ANY_ACCESS)

HANDLE I2C_handle;

int open()
{
	DWORD byte_return;

	I2C_handle = CreateFile(
		SPBTESTTOOL_USERMODE_PATH,
		(GENERIC_READ | GENERIC_WRITE),
		0,
		0,
		OPEN_EXISTING,
		FILE_FLAG_OVERLAPPED,
		0);

	if (I2C_handle == INVALID_HANDLE_VALUE)
	{
		printf("%s CreateFile error\n", __FUNCTION__);
		return 0;
	}

	printf("I2C CreateFile SUCCESS \n");

	if (!DeviceIoControl(
		I2C_handle,
		IOCTL_SPBTESTTOOL_OPEN,
		NULL,
		0,
		NULL,
		0,
		&byte_return,
		NULL))
	{
		printf("Failed to open I2c. %x\n", GetLastError());
		return 0;
	}

	printf("Open I2c SUCCESS \n");
	return 1;
}

int write(char *cmd, int cmd_len)
{
	if (I2C_handle == INVALID_HANDLE_VALUE)
		return 0;

	DWORD byte_write;

	if (!DeviceIoControl(I2C_handle,
		IOCTL_SPBTESTTOOL_WRITE,
		(LPVOID)cmd,
		(DWORD)cmd_len,
		NULL,
		0,
		&byte_write,
		NULL))
	{
		printf("Failed to write I2c. %x\n", GetLastError());
		return 0;
	}

	printf("I2C write %d byte\n", byte_write);

	return 1;
}

int read(char *cmd, int cmd_len, char* result, int result_len)
{
	if (I2C_handle == INVALID_HANDLE_VALUE)
		return 0;

	DWORD byte_read;

	printf("Start to read, cmd: %02x\n", *cmd);

	if (!DeviceIoControl(I2C_handle,
		IOCTL_SPBTESTTOOL_WRITEREAD,
		(LPVOID)cmd,
		(DWORD)cmd_len,
		(LPVOID)result,
		(DWORD)result_len,
		&byte_read,
		NULL))
	{
		printf("Failed to read I2c. %x\n", GetLastError());
		return 0;
	}

	printf("I2C read %d byte, Read Data :\n\n", byte_read);

	for (int i = 0; i < (int)byte_read; i++)
	{
		printf("%c", result[i]);
	}

	printf("\n");

	return 1;
}


void __cdecl main()
{
	if (!open())
	{
		printf("Open fail \n");
	}

	/* 
		(AIM8I)
		Send Command to MCU 
		Get MB Platform Name (0x50)
	*/
	char cmd[2];
	cmd[0] = 0x50;
	cmd[1] = 0x50;

	if (!write(cmd, 2))
	{
		printf("Write fail -%u \n", GetLastError());
	}


	/* Read from MCU */
	char result[24] = "\0";
	if (!read(cmd, 2, result, 11))
	{
		printf("Read fail - %u \n", GetLastError());
	}
}
